@@ -16,23 +16,23 @@
  * Boston, MA 021110-1307, USA.
  */
 
+#include "kerncompat.h"
 #include <stdio.h>
 #include <stdlib.h>
-#include <fcntl.h>
-#include <unistd.h>
 #include <getopt.h>
 #include <limits.h>
-
-#include "kerncompat.h"
+#include <errno.h>
+#include <string.h>
 #include "kernel-shared/ctree.h"
 #include "kernel-shared/volumes.h"
 #include "kernel-shared/disk-io.h"
-#include "kernel-shared/print-tree.h"
 #include "kernel-shared/transaction.h"
-#include "kernel-lib/list.h"
-#include "kernel-lib/radix-tree.h"
+#include "kernel-shared/extent_io.h"
 #include "common/utils.h"
 #include "common/help.h"
+#include "common/extent-cache.h"
+#include "common/messages.h"
+#include "common/string-utils.h"
 
 #define FIELD_BUF_LEN 80
 
@@ -54,8 +54,7 @@ static int debug_corrupt_block(struct extent_buffer *eb,
 				ret = -EIO;
 			if (ret < 0) {
 				errno = -ret;
-				error("cannot read eb bytenr %llu: %m",
-					(unsigned long long)eb->start);
+				error("cannot read eb bytenr %llu: %m", eb->start);
 				return ret;
 			}
 			printf("corrupting %llu copy %d\n", eb->start,
@@ -64,8 +63,7 @@ static int debug_corrupt_block(struct extent_buffer *eb,
 			ret = write_and_map_eb(eb->fs_info, eb);
 			if (ret < 0) {
 				errno = -ret;
-				error("cannot write eb bytenr %llu: %m",
-					(unsigned long long)eb->start);
+				error("cannot write eb bytenr %llu: %m", eb->start);
 				return ret;
 			}
 		}
@@ -86,24 +84,49 @@ static int debug_corrupt_block(struct extent_buffer *eb,
 static void print_usage(int ret)
 {
 	printf("usage: btrfs-corrupt-block [options] device\n");
-	printf("\t-l   Logical extent to be corrupted\n");
-	printf("\t-c   Copy of the extent to be corrupted (usually 1 or 2, default: 0)\n");
-	printf("\t-b   Number of bytes to be corrupted\n");
-	printf("\t-e   Extent to be corrupted\n");
-	printf("\t-E   The whole extent tree to be corrupted\n");
-	printf("\t-u   Given chunk item to be corrupted\n");
-	printf("\t-U   The whole chunk tree to be corrupted\n");
-	printf("\t-i   The inode item to corrupt (must also specify the field to corrupt)\n");
-	printf("\t-x   The file extent item to corrupt (must also specify -i for the inode and -f for the field to corrupt)\n");
-	printf("\t-m   The metadata block to corrupt (must also specify -f for the field to corrupt)\n");
-	printf("\t-K <u64,u8,u64> Corrupt the given key (must also specify -f for the field and optionally -r for the root)\n");
-	printf("\t-f   The field in the item to corrupt\n");
-	printf("\t-I <u64,u8,u64> Corrupt an item corresponding to the passed key triplet (must also specify the field to corrupt and root for the item)\n");
-	printf("\t-D <u64,u8,u64> Corrupt a dir item corresponding to the passed key triplet, must also specify a field\n");
-	printf("\t-d <u64,u8,u64> Delete item corresponding to passed key triplet\n");
-	printf("\t-r   Operate on this root\n");
-	printf("\t-C   Delete a csum for the specified bytenr.  When used with -b it'll delete that many bytes, otherwise it's just sectorsize\n");
-	printf("\t--block-group OFFSET  corrupt the given block group\n");
+	printf("\n");
+	printf("Corrupt data structures on a btrfs filesystem. For testing only!\n");
+	printf("\n");
+	printf("    -l EXTENT\n");
+	printf("    --logical EXTENT       logical extent to be corrupted\n");
+	printf("    -c COPY\n");
+	printf("    --copy COPY            copy of the extent to be corrupted (usually 1 or 2, default: 0)\n");
+	printf("    -b COUNT\n");
+	printf("    --bytes COUNT          number of bytes to be corrupted\n");
+	printf("    -e\n");
+	printf("    --extent-record        corrupt the extent\n");
+	printf("    -E\n");
+	printf("    --extent-tree          corrupt the whole extent tree\n");
+	printf("    -u\n");
+	printf("    --chunk-record         corrupt the given chunk\n");
+	printf("    -U\n");
+	printf("    --chunk-tree           corrupt the whole whole chunk tree\n");
+	printf("    -i INODE\n");
+	printf("    --inode INODE          inode number to corrupt (must also specify the field to corrupt)\n");
+	printf("    -x EXTENT\n");
+	printf("    --file-extent EXTENT   file extent item to corrupt (must also specify -i for the inode and -f for the field to corrupt)\n");
+	printf("    -m BLOCK\n");
+	printf("    --metadata-block BLOCK\n");
+	printf("                           metadata block to corrupt (must also specify -f for the field to corrupt)\n");
+	printf("    -k\n");
+	printf("    --keys                 corrupt block keys (set by --logical)\n");
+	printf("    -K <u64,u8,u64>\n");
+	printf("    --key <u64,u8,u64>     corrupt the given key (must also specify -f for the field and optionally -r for the root)\n");
+	printf("    -f FIELD\n");
+	printf("    --field FIELD          field name in the item to corrupt\n");
+	printf("    -I\n");
+	printf("    --item                 corrupt an item corresponding to the passed key triplet (must also specify the field, or a (bytes, offset, value) tuple to corrupt and root for the item)\n");
+	printf("    -D\n");
+	printf("    --dir-item             corrupt a dir item corresponding to the passed key triplet, must also specify a field\n");
+	printf("    -d\n");
+	printf("    --delete               delete item corresponding to passed key triplet\n");
+	printf("    -r\n");
+	printf("    --root                 operate on this root\n");
+	printf("    -C BYTENR\n");
+	printf("    --csum BYTENR          delete a csum for the specified bytenr.  When used with -b it'll delete that many bytes, otherwise it's just sectorsize\n");
+	printf("    --block-group OFFSET   corrupt the given block group\n");
+	printf("    --value VALUE          value to use for corrupting item data\n");
+	printf("    --offset OFFSET        offset to use for corrupting item data\n");
 	exit(ret);
 }
 
@@ -128,7 +151,7 @@ static void corrupt_keys(struct btrfs_trans_handle *trans,
 
 	fprintf(stderr,
 		"corrupting keys in block %llu slot %d swapping with %d\n",
-		(unsigned long long)eb->start, slot, bad_slot);
+		eb->start, slot, bad_slot);
 
 	if (btrfs_header_level(eb) == 0) {
 		btrfs_item_key(eb, &bad_key, bad_slot);
@@ -305,6 +328,7 @@ enum btrfs_inode_field {
 
 enum btrfs_file_extent_field {
 	BTRFS_FILE_EXTENT_DISK_BYTENR,
+	BTRFS_FILE_EXTENT_TYPE,
 	BTRFS_FILE_EXTENT_BAD,
 };
 
@@ -377,6 +401,8 @@ static enum btrfs_file_extent_field convert_file_extent_field(char *field)
 {
 	if (!strncmp(field, "disk_bytenr", FIELD_BUF_LEN))
 		return BTRFS_FILE_EXTENT_DISK_BYTENR;
+	if (!strncmp(field, "type", FIELD_BUF_LEN))
+		return BTRFS_FILE_EXTENT_TYPE;
 	return BTRFS_FILE_EXTENT_BAD;
 }
 
@@ -458,7 +484,7 @@ static int corrupt_block_group(struct btrfs_root *root, u64 bg, char *field)
 
 	corrupt_field = convert_block_group_field(field);
 	if (corrupt_field == BTRFS_BLOCK_GROUP_ITEM_BAD) {
-		fprintf(stderr, "Invalid field %s\n", field);
+		error("invalid field %s", field);
 		return -EINVAL;
 	}
 
@@ -469,9 +495,10 @@ static int corrupt_block_group(struct btrfs_root *root, u64 bg, char *field)
 	trans = btrfs_start_transaction(root, 1);
 	if (IS_ERR(trans)) {
 		btrfs_free_path(path);
-		fprintf(stderr, "Couldn't start transaction %ld\n",
-			PTR_ERR(trans));
-		return PTR_ERR(trans);
+		ret = PTR_ERR(trans);
+		errno = -ret;
+		error_msg(ERROR_MSG_START_TRANS, "%m");
+		return ret;
 	}
 
 	key.objectid = bg;
@@ -480,14 +507,14 @@ static int corrupt_block_group(struct btrfs_root *root, u64 bg, char *field)
 
 	ret = btrfs_search_slot(trans, root, &key, path, 0, 1);
 	if (ret < 0) {
-		fprintf(stderr, "Error searching for bg %llu %d\n", bg, ret);
+		error("error searching for bg %llu %d", bg, ret);
 		goto out;
 	}
 
 	ret = 0;
 	btrfs_item_key_to_cpu(path->nodes[0], &key, path->slots[0]);
 	if (key.type != BTRFS_BLOCK_GROUP_ITEM_KEY) {
-		fprintf(stderr, "Couldn't find the bg %llu\n", bg);
+		error("couldn't find the bg %llu", bg);
 		goto out;
 	}
 
@@ -530,7 +557,7 @@ static int corrupt_key(struct btrfs_root *root, struct btrfs_key *key,
 	int ret;
 
 	if (corrupt_field == BTRFS_KEY_BAD) {
-		fprintf(stderr, "Invalid field %s\n", field);
+		error("invalid field %s", field);
 		return -EINVAL;
 	}
 
@@ -548,7 +575,7 @@ static int corrupt_key(struct btrfs_root *root, struct btrfs_key *key,
 	if (ret < 0)
 		goto out;
 	if (ret > 0) {
-		fprintf(stderr, "Couldn't find the key to corrupt\n");
+		error("couldn't find the key to corrupt");
 		ret = -ENOENT;
 		goto out;
 	}
@@ -564,8 +591,7 @@ static int corrupt_key(struct btrfs_root *root, struct btrfs_key *key,
 		key->offset = generate_u64(key->objectid);
 		break;
 	default:
-		fprintf(stderr, "Invalid field %s, %d\n", field,
-			corrupt_field);
+		error("invalid field %s, %d", field, corrupt_field);
 		ret = -EINVAL;
 		goto out;
 	}
@@ -594,7 +620,7 @@ static int corrupt_dir_item(struct btrfs_root *root, struct btrfs_key *key,
 	int ret;
 
 	if (corrupt_field == BTRFS_DIR_ITEM_BAD) {
-		fprintf(stderr, "Invalid field %s\n", field);
+		error("invalid field %s", field);
 		return -EINVAL;
 	}
 
@@ -612,7 +638,7 @@ static int corrupt_dir_item(struct btrfs_root *root, struct btrfs_key *key,
 	if (ret) {
 		if (ret > 0)
 			ret = -ENOENT;
-		fprintf(stderr, "Error searching for dir item %d\n", ret);
+		error("error searching for dir item %d", ret);
 		goto out;
 	}
 
@@ -658,7 +684,7 @@ static int corrupt_inode(struct btrfs_trans_handle *trans,
 	int ret;
 
 	if (corrupt_field == BTRFS_INODE_FIELD_BAD) {
-		fprintf(stderr, "Invalid field %s\n", field);
+		error("invalid field %s", field);
 		return -EINVAL;
 	}
 
@@ -675,7 +701,7 @@ static int corrupt_inode(struct btrfs_trans_handle *trans,
 		goto out;
 	if (ret) {
 		if (!path->slots[0]) {
-			fprintf(stderr, "Couldn't find inode %llu\n", inode);
+			error("couldn't find inode %llu", inode);
 			ret = -ENOENT;
 			goto out;
 		}
@@ -685,7 +711,7 @@ static int corrupt_inode(struct btrfs_trans_handle *trans,
 
 	btrfs_item_key_to_cpu(path->nodes[0], &key, path->slots[0]);
 	if (key.objectid != inode) {
-		fprintf(stderr, "Couldn't find inode %llu\n", inode);
+		error("couldn't find inode %llu", inode);
 		ret = -ENOENT;
 		goto out;
 	}
@@ -750,19 +776,18 @@ out:
 
 static int corrupt_file_extent(struct btrfs_trans_handle *trans,
 			       struct btrfs_root *root, u64 inode, u64 extent,
-			       char *field)
+			       char *field, u64 bogus)
 {
 	struct btrfs_file_extent_item *fi;
 	struct btrfs_path *path;
 	struct btrfs_key key;
 	enum btrfs_file_extent_field corrupt_field;
-	u64 bogus;
 	u64 orig;
 	int ret = 0;
 
 	corrupt_field = convert_file_extent_field(field);
 	if (corrupt_field == BTRFS_FILE_EXTENT_BAD) {
-		fprintf(stderr, "Invalid field %s\n", field);
+		error("invalid field %s", field);
 		return -EINVAL;
 	}
 
@@ -778,7 +803,7 @@ static int corrupt_file_extent(struct btrfs_trans_handle *trans,
 	if (ret < 0)
 		goto out;
 	if (ret) {
-		fprintf(stderr, "Couldn't find extent %llu for inode %llu\n",
+		error("couldn't find extent %llu for inode %llu",
 			extent, inode);
 		ret = -ENOENT;
 		goto out;
@@ -789,9 +814,17 @@ static int corrupt_file_extent(struct btrfs_trans_handle *trans,
 	switch (corrupt_field) {
 	case BTRFS_FILE_EXTENT_DISK_BYTENR:
 		orig = btrfs_file_extent_disk_bytenr(path->nodes[0], fi);
-		bogus = generate_u64(orig);
+		bogus = (bogus == (u64)-1) ? generate_u64(orig) : bogus;
 		btrfs_set_file_extent_disk_bytenr(path->nodes[0], fi, bogus);
 		break;
+	case BTRFS_FILE_EXTENT_TYPE:
+		if (bogus == (u64)-1) {
+			error("specify a new extent type value (-v)");
+			ret = -EINVAL;
+			goto out;
+		}
+		btrfs_set_file_extent_type(path->nodes[0], fi, (u8)bogus);
+		break;
 	default:
 		ret = -EINVAL;
 		break;
@@ -811,8 +844,8 @@ static void shift_items(struct btrfs_root *root, struct extent_buffer *eb)
 	unsigned int data_end = btrfs_item_offset(eb, nritems - 1);
 
 	/* Shift the item data up to and including slot back by shift space */
-	memmove_extent_buffer(eb, btrfs_leaf_data(eb) + data_end - shift_space,
-			      btrfs_leaf_data(eb) + data_end,
+	memmove_extent_buffer(eb, btrfs_item_nr_offset(eb, 0) + data_end - shift_space,
+			      btrfs_item_nr_offset(eb, 0) + data_end,
 			      btrfs_item_offset(eb, slot - 1) - data_end);
 
 	/* Now update the item pointers. */
@@ -832,13 +865,13 @@ static int corrupt_metadata_block(struct btrfs_fs_info *fs_info, u64 block,
 
 	corrupt_field = convert_metadata_block_field(field);
 	if (corrupt_field == BTRFS_METADATA_BLOCK_BAD) {
-		fprintf(stderr, "Invalid field %s\n", field);
+		error("invalid field %s", field);
 		return -EINVAL;
 	}
 
 	eb = read_tree_block(fs_info, block, 0);
 	if (!extent_buffer_uptodate(eb)) {
-		fprintf(stderr, "Couldn't read in tree block %s\n", field);
+		error("couldn't read in tree block %s", field);
 		return -EINVAL;
 	}
 
@@ -850,12 +883,13 @@ static int corrupt_metadata_block(struct btrfs_fs_info *fs_info, u64 block,
 		u64 bogus = generate_u64(orig);
 
 		btrfs_set_header_generation(eb, bogus);
+		csum_tree_block_size(eb, fs_info->csum_size, 0,
+				     fs_info->csum_type);
 		ret = write_and_map_eb(fs_info, eb);
 		free_extent_buffer(eb);
 		if (ret < 0) {
 			errno = -ret;
-			fprintf(stderr,
-				"failed to write extent buffer at %llu: %m",
+			error("failed to write extent buffer at %llu: %m",
 				eb->start);
 			return ret;
 		}
@@ -884,8 +918,7 @@ static int corrupt_metadata_block(struct btrfs_fs_info *fs_info, u64 block,
 
 		root = btrfs_read_fs_root(fs_info, &root_key);
 		if (IS_ERR(root)) {
-			fprintf(stderr, "Couldn't find owner root %llu\n",
-				key.objectid);
+			error("couldn't find owner root %llu", key.objectid);
 			return PTR_ERR(root);
 		}
 
@@ -896,15 +929,16 @@ static int corrupt_metadata_block(struct btrfs_fs_info *fs_info, u64 block,
 		trans = btrfs_start_transaction(root, 1);
 		if (IS_ERR(trans)) {
 			btrfs_free_path(path);
-			fprintf(stderr, "Couldn't start transaction %ld\n",
-				PTR_ERR(trans));
-			return PTR_ERR(trans);
+			ret = PTR_ERR(trans);
+			errno = -ret;
+			error_msg(ERROR_MSG_START_TRANS, "%m");
+			return ret;
 		}
 
 		path->lowest_level = level;
 		ret = btrfs_search_slot(trans, root, &key, path, 0, 1);
 		if (ret < 0) {
-			fprintf(stderr, "Error searching to node %d\n", ret);
+			error("error searching to node %d", ret);
 			btrfs_free_path(path);
 			btrfs_abort_transaction(trans, ret);
 			return ret;
@@ -934,7 +968,7 @@ static int corrupt_btrfs_item(struct btrfs_root *root, struct btrfs_key *key,
 
 	corrupt_field = convert_item_field(field);
 	if (corrupt_field == BTRFS_ITEM_BAD) {
-		fprintf(stderr, "Invalid field %s\n", field);
+		error("invalid field %s", field);
 		return -EINVAL;
 	}
 
@@ -945,14 +979,15 @@ static int corrupt_btrfs_item(struct btrfs_root *root, struct btrfs_key *key,
 	trans = btrfs_start_transaction(root, 1);
 	if (IS_ERR(trans)) {
 		btrfs_free_path(path);
-		fprintf(stderr, "Couldn't start transaction %ld\n",
-			PTR_ERR(trans));
-		return PTR_ERR(trans);
+		ret = PTR_ERR(trans);
+		errno = -ret;
+		error_msg(ERROR_MSG_START_TRANS, "%m");
+		return ret;
 	}
 
 	ret = btrfs_search_slot(trans, root, key, path, 0, 1);
 	if (ret != 0) {
-		fprintf(stderr, "Error searching to node %d\n", ret);
+		error("error searching to node %d", ret);
 		goto out;
 	}
 
@@ -974,6 +1009,57 @@ out:
 	return ret;
 }
 
+static int corrupt_btrfs_item_data(struct btrfs_root *root,
+				   struct btrfs_key *key,
+				   u64 bogus_offset, u64 bogus_size,
+				   char bogus_value)
+{
+	struct btrfs_trans_handle *trans;
+	struct btrfs_path *path;
+	int ret;
+	void *data;
+	struct extent_buffer *leaf;
+	int slot;
+	u32 item_size;
+
+	path = btrfs_alloc_path();
+	if (!path)
+		return -ENOMEM;
+
+	trans = btrfs_start_transaction(root, 1);
+	if (IS_ERR(trans)) {
+		ret = PTR_ERR(trans);
+		errno = -ret;
+		error_msg(ERROR_MSG_START_TRANS, "%m");
+		goto free_path;
+	}
+
+	ret = btrfs_search_slot(trans, root, key, path, 0, 1);
+	if (ret != 0) {
+		error("error searching to node %d", ret);
+		goto commit_txn;
+	}
+	leaf = path->nodes[0];
+	slot = path->slots[0];
+	data = btrfs_item_ptr(leaf, slot, void);
+	item_size = btrfs_item_size(leaf, slot);
+	if (bogus_offset + bogus_size > item_size) {
+		error("item corruption past end of item: %llu > %u\n",
+			bogus_offset + bogus_size, item_size);
+		ret = -EINVAL;
+		goto commit_txn;
+	}
+	data += bogus_offset;
+	memset_extent_buffer(leaf, bogus_value, (unsigned long)data, bogus_size);
+	btrfs_mark_buffer_dirty(leaf);
+
+commit_txn:
+	btrfs_commit_transaction(trans, root);
+free_path:
+	btrfs_free_path(path);
+	return ret;
+}
+
 static int delete_item(struct btrfs_root *root, struct btrfs_key *key)
 {
 	struct btrfs_trans_handle *trans;
@@ -987,16 +1073,17 @@ static int delete_item(struct btrfs_root *root, struct btrfs_key *key)
 	trans = btrfs_start_transaction(root, 1);
 	if (IS_ERR(trans)) {
 		btrfs_free_path(path);
-		fprintf(stderr, "Couldn't start transaction %ld\n",
-			PTR_ERR(trans));
-		return PTR_ERR(trans);
+		ret = PTR_ERR(trans);
+		errno = -ret;
+		error_msg(ERROR_MSG_START_TRANS, "%m");
+		return ret;
 	}
 
 	ret = btrfs_search_slot(trans, root, key, path, -1, 1);
 	if (ret) {
 		if (ret > 0)
 			ret = -ENOENT;
-		fprintf(stderr, "Error searching to node %d\n", ret);
+		error("error searching to node %d", ret);
 		goto out;
 	}
 	ret = btrfs_del_item(trans, root, path);
@@ -1015,14 +1102,15 @@ static int delete_csum(struct btrfs_root *root, u64 bytenr, u64 bytes)
 	root = btrfs_csum_root(root->fs_info, bytenr);
 	trans = btrfs_start_transaction(root, 1);
 	if (IS_ERR(trans)) {
-		fprintf(stderr, "Couldn't start transaction %ld\n",
-			PTR_ERR(trans));
-		return PTR_ERR(trans);
+		ret = PTR_ERR(trans);
+		errno = -ret;
+		error_msg(ERROR_MSG_START_TRANS, "%m");
+		return ret;
 	}
 
 	ret = btrfs_del_csums(trans, bytenr, bytes);
 	if (ret)
-		fprintf(stderr, "Error deleting csums %d\n", ret);
+		error("error deleting csums %d", ret);
 	btrfs_commit_transaction(trans, root);
 	return ret;
 }
@@ -1056,11 +1144,11 @@ static int corrupt_item_nocow(struct btrfs_trans_handle *trans,
 	}
 	btrfs_item_key_to_cpu(leaf, &key, slot);
 	if (del) {
-		fprintf(stdout, "Deleting key and data [%llu, %u, %llu].\n",
+		fprintf(stdout, "Deleting key and data [%llu, %u, %llu]\n",
 			key.objectid, key.type, key.offset);
 		btrfs_del_item(trans, root, path);
 	} else {
-		fprintf(stdout, "Corrupting key and data [%llu, %u, %llu].\n",
+		fprintf(stdout, "Corrupting key and data [%llu, %u, %llu]\n",
 			key.objectid, key.type, key.offset);
 		ptr = btrfs_item_ptr_offset(leaf, slot);
 		item_size = btrfs_item_size(leaf, slot);
@@ -1098,7 +1186,7 @@ static int corrupt_chunk_tree(struct btrfs_trans_handle *trans,
 	ret = btrfs_search_slot(trans, root, &key, path, 0, 0);
 	BUG_ON(ret == 0);
 	if (ret < 0) {
-		fprintf(stderr, "Error searching tree\n");
+		error("error searching tree");
 		goto free_out;
 	}
 	/* corrupt/del dev_item first */
@@ -1125,7 +1213,7 @@ static int corrupt_chunk_tree(struct btrfs_trans_handle *trans,
 	ret = btrfs_search_slot(trans, root, &key, path, 0, 0);
 	BUG_ON(ret == 0);
 	if (ret < 0) {
-		fprintf(stderr, "Error searching tree\n");
+		error("error searching tree");
 		goto free_out;
 	}
 	/* corrupt/del chunk then*/
@@ -1161,12 +1249,11 @@ static int find_chunk_offset(struct btrfs_root *root,
 	 */
 	ret = btrfs_search_slot(NULL, root, &key, path, 0, 0);
 	if (ret > 0) {
-		fprintf(stderr, "Can't find chunk with given offset %llu\n",
-			offset);
+		error("can't find chunk with given offset %llu", offset);
 		goto out;
 	}
 	if (ret < 0) {
-		fprintf(stderr, "Error searching chunk\n");
+		error("error searching chunk");
 		goto out;
 	}
 out:
@@ -1179,7 +1266,7 @@ static void parse_key(u64 *objectid, u8 *type, u64 *offset)
 
 	int ret = sscanf(optarg, "%llu,%hhu,%llu", objectid, type, offset);
 	if (ret != 3) {
-	        fprintf(stderr, "error parsing key '%s': %d\n", optarg, errno);
+	        error("error parsing key '%s': %d", optarg, errno);
 	        print_usage(1);
 	}
 }
@@ -1197,7 +1284,7 @@ static struct btrfs_root *open_root(struct btrfs_fs_info *fs_info,
 
 	root = btrfs_read_fs_root(fs_info, &root_key);
 	if (IS_ERR(root)) {
-		fprintf(stderr, "couldn't find root %llu\n", root_objectid);
+		error("couldn't find root %llu", root_objectid);
 		print_usage(1);
 	}
 
@@ -1230,13 +1317,17 @@ int main(int argc, char **argv)
 	u64 csum_bytenr = 0;
 	u64 block_group = 0;
 	char field[FIELD_BUF_LEN];
+	u64 bogus_value = (u64)-1;
+	u64 bogus_offset = (u64)-1;
 
 	field[0] = '\0';
 	memset(&key, 0, sizeof(key));
 
 	while(1) {
 		int c;
-		enum { GETOPT_VAL_BLOCK_GROUP = 256 };
+		enum { GETOPT_VAL_BLOCK_GROUP = GETOPT_VAL_FIRST,
+			GETOPT_VAL_VALUE, GETOPT_VAL_OFFSET,
+		};
 		static const struct option long_options[] = {
 			/* { "byte-count", 1, NULL, 'b' }, */
 			{ "logical", required_argument, NULL, 'l' },
@@ -1255,9 +1346,11 @@ int main(int argc, char **argv)
 			{ "item", no_argument, NULL, 'I'},
 			{ "dir-item", no_argument, NULL, 'D'},
 			{ "delete", no_argument, NULL, 'd'},
-			{ "root", no_argument, NULL, 'r'},
+			{ "root", required_argument, NULL, 'r'},
 			{ "csum", required_argument, NULL, 'C'},
 			{ "block-group", required_argument, NULL, GETOPT_VAL_BLOCK_GROUP},
+			{ "value", required_argument, NULL, GETOPT_VAL_VALUE},
+			{ "offset", required_argument, NULL, GETOPT_VAL_OFFSET},
 			{ "help", no_argument, NULL, GETOPT_VAL_HELP},
 			{ NULL, 0, NULL, 0 }
 		};
@@ -1328,6 +1421,12 @@ int main(int argc, char **argv)
 			case GETOPT_VAL_BLOCK_GROUP:
 				block_group = arg_strtou64(optarg);
 				break;
+			case GETOPT_VAL_VALUE:
+				bogus_value = arg_strtou64(optarg);
+				break;
+			case GETOPT_VAL_OFFSET:
+				bogus_offset = arg_strtou64(optarg);
+				break;
 			case GETOPT_VAL_HELP:
 			default:
 				print_usage(c != GETOPT_VAL_HELP);
@@ -1338,12 +1437,11 @@ int main(int argc, char **argv)
 		return 1;
 	dev = argv[optind];
 
-	radix_tree_init();
 	cache_tree_init(&root_cache);
 
 	root = open_ctree(dev, 0, OPEN_CTREE_WRITES);
 	if (!root) {
-		fprintf(stderr, "Open ctree failed\n");
+		error("open ctree failed");
 		exit(1);
 	}
 	target_root = root;
@@ -1383,7 +1481,7 @@ int main(int argc, char **argv)
 		del = rand_range(3);
 		path = btrfs_alloc_path();
 		if (!path) {
-			fprintf(stderr, "path allocation failed\n");
+			error_msg(ERROR_MSG_MEMORY, NULL);
 			goto out_close;
 		}
 
@@ -1397,7 +1495,7 @@ int main(int argc, char **argv)
 		ret = corrupt_item_nocow(trans, root->fs_info->chunk_root,
 					 path, del);
 		if (ret < 0)
-			fprintf(stderr, "Failed to corrupt chunk record\n");
+			error("failed to corrupt chunk record");
 		btrfs_commit_transaction(trans, root);
 		goto out_close;
 	}
@@ -1408,7 +1506,7 @@ int main(int argc, char **argv)
 		BUG_ON(IS_ERR(trans));
 		ret = corrupt_chunk_tree(trans, root->fs_info->chunk_root);
 		if (ret < 0)
-			fprintf(stderr, "Failed to corrupt chunk tree\n");
+			error("failed to corrupt chunk tree");
 		btrfs_commit_transaction(trans, root);
 		goto out_close;
 	}
@@ -1424,9 +1522,8 @@ int main(int argc, char **argv)
 			printf("corrupting inode\n");
 			ret = corrupt_inode(trans, root, inode, field);
 		} else {
-			printf("corrupting file extent\n");
 			ret = corrupt_file_extent(trans, root, inode,
-						  file_extent, field);
+						  file_extent, field, bogus_value);
 		}
 		btrfs_commit_transaction(trans, root);
 		goto out_close;
@@ -1454,7 +1551,16 @@ int main(int argc, char **argv)
 		if (!root_objectid)
 			print_usage(1);
 
-		ret = corrupt_btrfs_item(target_root, &key, field);
+		if (*field != 0)
+			ret = corrupt_btrfs_item(target_root, &key, field);
+		else if (bogus_offset != (u64)-1 &&
+			 bytes != (u64)-1 &&
+			 bogus_value != (u64)-1)
+			ret = corrupt_btrfs_item_data(target_root, &key,
+						      bogus_offset, bytes,
+						      bogus_value);
+		else
+			print_usage(1);
 		goto out_close;
 	}
 	if (delete) {
@@ -1501,9 +1607,9 @@ int main(int argc, char **argv)
 			eb = btrfs_find_create_tree_block(root->fs_info,
 					logical);
 			if (!eb) {
-				error(
-		"not enough memory to allocate extent buffer for bytenr %llu",
-					(unsigned long long)logical);
+				error_msg(ERROR_MSG_MEMORY,
+					"allocating extent buffer for bytenr %llu",
+					logical);
 				ret = 1;
 				goto out_close;
 			}
