@@ -16,21 +16,24 @@
  * Boston, MA 021110-1307, USA.
  */
 
+#include "kerncompat.h"
 #include <stdio.h>
 #include <stdlib.h>
 #include <fcntl.h>
 #include <unistd.h>
 #include <getopt.h>
-#include "kerncompat.h"
+#include <errno.h>
+#include <string.h>
+#include "kernel-lib/sizes.h"
 #include "kernel-shared/ctree.h"
+#include "kernel-shared/extent_io.h"
 #include "kernel-shared/volumes.h"
 #include "kernel-shared/disk-io.h"
-#include "kernel-shared/print-tree.h"
-#include "kernel-shared/transaction.h"
-#include "kernel-lib/list.h"
-#include "kernel-lib/radix-tree.h"
+#include "common/internal.h"
 #include "common/utils.h"
 #include "common/help.h"
+#include "common/extent-cache.h"
+#include "common/string-utils.h"
 
 #define BUFFER_SIZE SZ_64K
 
@@ -178,8 +181,7 @@ static int write_extent_content(struct btrfs_fs_info *fs_info, int out_fd,
 					  mirror);
 		if (ret < 0) {
 			errno = -ret;
-			fprintf(stderr,
-				"Failed to read extent at [%llu, %llu]: %m\n",
+			error("failed to read extent at [%llu, %llu]: %m",
 				logical, logical + length);
 			return ret;
 		}
@@ -188,7 +190,7 @@ static int write_extent_content(struct btrfs_fs_info *fs_info, int out_fd,
 			if (ret > 0)
 				ret = -EINTR;
 			errno = -ret;
-			fprintf(stderr, "output file write failed: %m\n");
+			error("output file write failed: %m");
 			return ret;
 		}
 		cur_offset += cur_len;
@@ -261,12 +263,11 @@ int main(int argc, char **argv)
 
 	dev = argv[optind];
 
-	radix_tree_init();
 	cache_tree_init(&root_cache);
 
 	root = open_ctree(dev, 0, 0);
 	if (!root) {
-		fprintf(stderr, "Open ctree failed\n");
+		error("open ctree failed");
 		free(output_file);
 		exit(1);
 	}
@@ -299,7 +300,7 @@ int main(int argc, char **argv)
 	ret = map_one_extent(root->fs_info, &cur_logical, &cur_len, 0);
 	if (ret < 0) {
 		errno = -ret;
-		fprintf(stderr, "Failed to find extent at [%llu,%llu): %m\n",
+		error("failed to find extent at [%llu,%llu): %m",
 			cur_logical, cur_logical + cur_len);
 		goto out_close_fd;
 	}
@@ -312,14 +313,12 @@ int main(int argc, char **argv)
 		ret = map_one_extent(root->fs_info, &cur_logical, &cur_len, 1);
 		if (ret < 0) {
 			errno = -ret;
-			fprintf(stderr,
-				"Failed to find extent at [%llu,%llu): %m\n",
+			error("Failed to find extent at [%llu,%llu): %m",
 				cur_logical, cur_logical + cur_len);
 			goto out_close_fd;
 		}
 		if (ret > 0) {
-			fprintf(stderr,
-				"Failed to find any extent at [%llu,%llu)\n",
+			error("failed to find any extent at [%llu,%llu)",
 				cur_logical, cur_logical + cur_len);
 			goto out_close_fd;
 		}
@@ -359,7 +358,7 @@ int main(int argc, char **argv)
 	}
 
 	if (!found) {
-		fprintf(stderr, "No extent found at range [%llu,%llu)\n",
+		error("no extent found at range [%llu,%llu)",
 			logical, logical + bytes);
 	}
 out_close_fd:
